@@ -27,11 +27,8 @@ redist.county.relabel <- function(adj, counties,  simplify = TRUE) {
         counties <- as.character(counties)
     }
 
-    groups <- rep(0, length(counties))
     sorted <- sort(unique(counties))
-    for (i in 1:length(counties)) {
-        groups[i] <- which(sorted == counties[i])
-    }
+    groups <- match(counties, sorted)
 
     component <- data.frame(counties = counties, comp = contiguity(adj, groups)) %>%
         group_by(counties) %>%
@@ -41,9 +38,9 @@ redist.county.relabel <- function(adj, counties,  simplify = TRUE) {
         mutate(countiescomp = ifelse(.data$comps > 1, paste0(counties, "-", .data$comp), counties)) %>%
         ungroup()
     if (simplify) {
-        return(redist.county.id(component$countiescomp))
+        redist.county.id(component$countiescomp)
     } else {
-        return(component$countiescomp)
+        component$countiescomp
     }
 }
 
@@ -72,5 +69,5 @@ redist.county.id <- function(counties) {
         cli_abort("{.arg counties} must be a character, numeric, or integer vector.")
     }
 
-    return(county_id)
+    county_id
 }
