@@ -71,6 +71,12 @@ summary.redist_plans <- function(object, district = 1L, all_runs = TRUE, vi_max
     if (is.null(n_distr)) n_distr <- max(plans_m[, 1])
 
     fmt_comma <- function(x) format(x, nsmall = 0, digits = 1, big.mark = ",")
+    if (n_distr == 1 || nrow(plans_m) == 1) {
+        cli_text("{fmt_comma(n_samp)}{cli::qty(n_samp)} sampled plan{?s} of
+                 {n_distr} district{?s} on
+                 {fmt_comma(nrow(plans_m))}{cli::qty(nrow(plans_m))} unit{?s}")
+        return(invisible(1))
+    }
 
     prec_pop <- attr(object, "prec_pop")
     if (is.null(prec_pop)) {
@@ -104,7 +110,8 @@ summary.redist_plans <- function(object, district = 1L, all_runs = TRUE, vi_max
 
             const_cols <- vapply(addl_cols, function(col) {
                 x <- object[[col]][idx]
-                all(is.na(x)) || all(x == x[1])
+                all(is.na(x)) || all(x == x[1]) ||
+                    any(tapply(x, object[['chain']][idx], FUN = function(z) length(unique(z))) == 1)
             }, numeric(1))
             addl_cols <- addl_cols[!const_cols]
 
@@ -128,7 +135,6 @@ summary.redist_plans <- function(object, district = 1L, all_runs = TRUE, vi_max
 
         }
 
-
         run_dfs <- list()
         n_runs <- length(all_diagn)
         warn_bottlenecks <- FALSE
@@ -230,6 +236,13 @@ summary.redist_plans <- function(object, district = 1L, all_runs = TRUE, vi_max
                 chain <- rep(1, nrow(object))
             }
 
+            const_cols <- vapply(addl_cols, function(col) {
+                x <- object[[col]][idx]
+                all(is.na(x)) || all(x == x[1]) ||
+                    any(tapply(x, object[['chain']][idx], FUN = function(z) length(unique(z))) == 1)
+            }, numeric(1))
+            addl_cols <- addl_cols[!const_cols]
+
             rhats <- vapply(addl_cols, function(col) {
                 x <- object[[col]][idx]
                 na_omit <- !is.na(x)
@@ -250,7 +263,9 @@ summary.redist_plans <- function(object, district = 1L, all_runs = TRUE, vi_max
             }
             cat("\n")
         } else {
-            out <- NULL
+            out <- tibble(accept_rate = attr(object, "mh_acceptance"),
+                          div_q10 = div_rg[1],
+                          div_q90 = div_rg[2])
         }
 
         cli::cli_li(cli::col_grey("
