@@ -14,34 +14,48 @@
 #' criteria. This implements the ideas in the below-referenced paper, "Voting
 #' Rights, Markov Chains, and Optimization by Short Bursts."
 #'
-#' @param map A \code{\link{redist_map}} object.
+#' @param map A [redist_map] object.
 #' @param score_fn A function which takes a matrix of plans and returns a score
-#' for each plan. Can also be a purrr-style anonymous function. See
-#' [`?scorers`][scorers] for some function factories for common scoring rules.
-#' @param stop_at A threshold to stop optimization at.
-#' @param burst_size The size of each burst. 10 is recommended for mergesplit and 50 for flip.
+#'   (or, generally, a row vector) for each plan. Can also be a purrr-style
+#'   anonymous function. See [`?scorers`][scorers] for some function factories
+#'   for common scoring rules.
+#' @param stop_at A threshold to stop optimization at. When `score_fn` returns a
+#'   row vector per plan, `maximize` can be an equal-length vector specifying a
+#'   threshold for each dimension, which must all be met for the algorithm to
+#'   stop.
+#' @param burst_size The size of each burst. 10 is recommended for the
+#'   `mergesplit` backend and 50 for the `flip` backend. Can also provide
+#'   burst schedule function which takes the current iteration (an integer)
+#'   and returns the desired burst size. This can be a random function.
 #' @param max_bursts The maximum number of bursts to run before returning.
-#' @param maximize If \code{TRUE}, try to maximize the score; otherwise, try to
-#' minimize it.
+#' @param maximize If `TRUE`, try to maximize the score; otherwise, try to
+#'   minimize it. When `score_fn` returns a row vector per plan, `maximize` can
+#'   be an equal-length vector specifying whether each dimension should be
+#'   maximized or minimized.
 #' @param init_plan The initial state of the map. If not provided, will default to
-#' the reference map of the \code{map} object, or if none exists, will sample
-#' a random initial state using \code{\link{redist_smc}}. You can also request
-#' a random initial state by setting \code{init_plan="sample"}.
+#' the reference map of the `map` object, or if none exists, will sample
+#' a random initial state using [redist_smc()]. You can also request
+#' a random initial state by setting `init_plan="sample"`.
 #' @param counties A vector containing county (or other administrative or
 #' geographic unit) labels for each unit, which may be integers ranging from 1
 #' to the number of counties, or a factor or character vector.  If provided, the
-#' algorithm will only generate maps which split up to \code{ndists-1} counties.
+#' algorithm will only generate maps which split up to `ndists-1` counties.
 #' If no county-split constraint is desired, this parameter should be left blank.
 #' @param constraints A `redist_constr` with Gibbs constraints.
 #' @param compactness Controls the compactness of the generated districts, with
 #' higher values preferring more compact districts. Must be non-negative. See
 #' \code{\link{redist_mergesplit}} for more information.
 #' @param adapt_k_thresh The threshold value used in the heuristic to select a
-#' value \code{k_i} for each splitting iteration. Set to 0.9999 or 1 if
-#' the algorithm does not appear to be sampling from the target distribution.
-#' Must be between 0 and 1.
-#' @param return_all Whether to return all the
-#' Recommended for monitoring purposes.
+#' value `k_i` for each splitting iteration.
+#' @param reversible If `FALSE` and `backend="mergesplit"`, the Markov chain
+#' used will not be reversible. This may speed up optimization.
+#' @param fixed_k If not `NULL`, will be used to set the `k` parameter for the
+#'   `mergesplit` backend. If e.g. `k=1` then the best edge in each spanning
+#'   tree will be used.  Lower values may speed up optimization at the
+#'   cost of the Markov chain no longer targeting a known distribution.
+#'   Recommended only in conjunction with `reversible=FALSE`.
+#' @param return_all Whether to return all the burst results or just the best
+#' one (generally, the Pareto frontier). Recommended for monitoring purposes.
 #' @param thin Save every `thin`-th sample. Defaults to no thinning (1). Ignored
 #' if `return_all=TRUE`.
 #' @param backend the MCMC algorithm to use within each burst, either
@@ -76,6 +90,7 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
                               max_bursts = 500L, maximize = TRUE, init_plan = NULL,
                               counties = NULL,  constraints = redist_constr(map),
                               compactness = 1, adapt_k_thresh = 0.95,
+                              reversible=TRUE, fixed_k=NULL,
                               return_all = TRUE, thin = 1L, backend = "mergesplit",
                               flip_lambda = 0, flip_eprob = 0.05,
                               verbose = TRUE) {
@@ -86,14 +101,17 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
     ndists <- attr(map, "ndists")
     thin <- as.integer(thin)
 
-    burst_size <- as.integer(burst_size)
+    if (!is.function(burst_size)) {
+        per_burst <- as.integer(burst_size)
+        burst_size <- function(i) per_burst
+    }
     max_bursts <- as.integer(max_bursts)
     match.arg(backend, c("flip", "mergesplit"))
 
     score_fn <- rlang::as_closure(score_fn)
     stopifnot(is.function(score_fn))
     if (!is.numeric(stop_at)) {
-        stop_at <- if (maximize) Inf else -Inf
+        stop_at <- Inf
     }
 
     if (compactness < 0)
@@ -101,7 +119,7 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
     if (adapt_k_thresh < 0 | adapt_k_thresh > 1)
         cli_abort("{.arg adapt_k_thresh} must lie in [0, 1].")
 
-    if (burst_size < 1 || max_bursts < 1)
+    if (burst_size(1) < 1 || max_bursts < 1)
         cli_abort("{.arg burst_size} and {.arg max_bursts} must be positive.")
     if (thin < 1 || thin > max_bursts)
         cli_abort("{.arg thin} must be a positive integer, and no larger than {.arg max_bursts}.")
@@ -114,7 +132,7 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
             cli_abort("County vector must not contain missing values.")
 
         # handle discontinuous counties
-        component <- contiguity(adj, as.integer(as.factor(counties)))
+        component <- contiguity(adj, vctrs::vec_group_id(counties))
         counties <- dplyr::if_else(component > 1,
             paste0(as.character(counties), "-", component),
             as.character(counties)) %>%
@@ -126,7 +144,7 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
         }
     }
 
-    if (is.null(init_plan)) init_plan <- as.integer(as.factor(get_existing(map)))
+    if (is.null(init_plan)) init_plan <- vctrs::vec_group_id(get_existing(map))
     if (length(init_plan) == 0L || isTRUE(init_plan == "sample")) {
         init_plan <- as.integer(get_plans_matrix(
             redist_smc(map, 10, counties, resample = FALSE, ref_name = FALSE, silent = TRUE, ncores = 1))[, 1])
@@ -159,24 +177,29 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
     constraints <- as.list(constraints)
 
     if (backend == "mergesplit") {
-        # kind of hacky -- extract k=... from outupt
-        if (!requireNamespace("utils", quietly = TRUE)) stop()
-        out <- utils::capture.output({
-            x <- ms_plans(1, adj, init_plan, counties, pop, ndists, pop_bounds[2],
-                pop_bounds[1], pop_bounds[3], compactness,
-                list(), adapt_k_thresh, 0L, 1L, verbosity = 3)
-        }, type = "output")
-        rm(x)
-        k <- as.integer(stats::na.omit(stringr::str_match(out, "Using k = (\\d+)")[, 2]))
-        if (length(k) == 0)
-            cli_abort(c("Adaptive {.var k} not found. This error should not happen.",
-                ">" = "Please file an issue at
-                        {.url https://github.com/alarm-redist/redist/issues/new}"))
+        control = list(adapt_k_thresh=adapt_k_thresh, do_mh=reversible)
+        if (is.null(fixed_k)) {
+            # kind of hacky -- extract k=... from outupt
+            if (!requireNamespace("utils", quietly = TRUE)) stop()
+            out <- utils::capture.output({
+                x <- ms_plans(1, adj, init_plan, counties, pop, ndists, pop_bounds[2],
+                    pop_bounds[1], pop_bounds[3], compactness,
+                    list(), control, 0L, 1L, verbosity = 3)
+            }, type = "output")
+            rm(x)
+            k <- as.integer(stats::na.omit(stringr::str_match(out, "Using k = (\\d+)")[, 2]))
+            if (length(k) == 0)
+                cli_abort(c("Adaptive {.var k} not found. This error should not happen.",
+                    ">" = "Please file an issue at
+                            {.url https://github.com/alarm-redist/redist/issues/new}"))
+        } else {
+            k = fixed_k
+        }
 
-        run_burst <- function(init) {
-            ms_plans(burst_size, adj, init, counties, pop, ndists,
+        run_burst <- function(init, steps) {
+            ms_plans(steps, adj, init, counties, pop, ndists,
                 pop_bounds[2], pop_bounds[1], pop_bounds[3], compactness,
-                constraints, 1.0, k, 1L, verbosity = 0)$plans[, -1L]
+                constraints, control, k, 1L, verbosity = 0)$plans[, -1L]
         }
     } else {
 
@@ -187,9 +210,9 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
             cli_abort("{.arg flip_lambda} must be a nonnegative integer.")
         }
 
-        run_burst <- function(init) {
+        run_burst <- function(init, steps) {
             skinny_flips(adj = adj, init_plan = init, total_pop = pop,
-                pop_tol = pop_tol, nsims = burst_size,
+                pop_tol = pop_tol, nsims = steps,
                 eprob = flip_eprob, lambda = flip_lambda,
                 constraints = constraints)
         }
@@ -199,20 +222,41 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
     burst <- 1
     n_out <- max_bursts %/% thin
     out_mat <- matrix(0L, nrow = V, ncol = n_out)
-    cur_best <- init_plan
+    burst_sizes <- integer(n_out)
+    cur_best <- matrix(init_plan, ncol=1)
+    rescale <- 1 - maximize * 2
 
-    scores <- numeric(n_out)
-    cur_best_score <- score_fn(matrix(init_plan, ncol = 1))
+    cur_best_scores <- score_fn(matrix(init_plan, ncol = 1))
+    if (!is.matrix(cur_best_scores)) {
+        cur_best_scores = matrix(cur_best_scores, ncol=1)
+        rownames(cur_best_scores) = "score"
+    } else {
+        cur_best_scores = t(cur_best_scores)
+        if (!is.null(names(rescale))) {
+            rescale = rescale[match(rownames(cur_best_scores), names(rescale))]
+        }
+    }
+    cur_best_scores <- cur_best_scores * rescale
+    dim_score <- nrow(cur_best_scores)
+
+    scores <- matrix(nrow=n_out, ncol=dim_score)
+    colnames(scores) = rownames(cur_best_scores)
 
     if (verbose) {
+        fmt_score <- function(x) {
+            paste0(sprintf("%f", x * rescale), collapse=" ")
+        }
         if (backend == "mergesplit") {
             cat("MERGE-SPLIT SHORT BURSTS\n")
         } else {
             cat("FLIP SHORT BURSTS\n")
         }
-        cat("Sampling up to", max_bursts, "bursts of", burst_size,
+        cat("Sampling up to", max_bursts, "bursts of", burst_size(1),
             "iterations each.\n")
-        cat("Burst  Improve?  Score\n")
+        cat("Burst  Improve? ")
+        cur_fmt_len <- nchar(fmt_score(cur_best_scores[, 1]))
+        cols <- stringr::str_pad(colnames(scores), round(cur_fmt_len / dim_score))
+        cat(cols, "\n")
     }
     report_int <- max(round(max_bursts/10), 1)
     improve_ch <- sample(c("\U0001F973", "\U0001F600", "\U0001F60E",
@@ -221,64 +265,93 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
         "\U0001F4A5", "\U0001F389", "\U26C4",
         "\U0001F31F", "\U0001F308"))
     improve_ct <- 1L
-    keep <- seq_len(burst_size)
     idx <- 1L
+    converged <- FALSE
     for (burst in 1:max_bursts) {
-        plans <- run_burst(cur_best)[, keep]
-        plan_scores <- score_fn(plans)
-
-        if (maximize) {
-            best_idx <- which.max(plan_scores)
-            best_score <- plan_scores[best_idx]
-            condition <- best_score > cur_best_score
-        } else {
-            best_idx <- which.min(plan_scores)
-            best_score <- plan_scores[best_idx]
-            condition <- best_score < cur_best_score
+        this_burst_size <- burst_size(burst)
+        burst_sizes[burst] <- this_burst_size
+        if (this_burst_size <= 0) {
+            cli_abort(c("Burst size must be at least 1.",
+                        "x"="Found {this_burst_size} on iteration {burst}."))
         }
+        keep <- seq_len(this_burst_size)
+        burst_init = cur_best[, sample.int(ncol(cur_best), 1)]
+        plans <- run_burst(burst_init, this_burst_size)[, keep, drop=FALSE]
+        plan_scores <- t(matrix(score_fn(plans), ncol=dim_score))
+        plan_scores <- plan_scores * rescale
 
-        if (condition) { # improvement
-            cur_best <- plans[, best_idx]
-            cur_best_score <- best_score
+        cur_best <- cbind(cur_best, plans)
+        cur_best_scores <- cbind(cur_best_scores, plan_scores)
+
+        dominated <- pareto_dominated(cur_best_scores)
+        improved <- any(!tail(dominated, this_burst_size))
+        # remove dominated plans
+        cur_best <- cur_best[, !dominated, drop=FALSE]
+        cur_best_scores <- cur_best_scores[, !dominated, drop=FALSE]
+
+        # add new undominated plans
+        out_idx = sample.int(ncol(cur_best), 1) # random plan from frontier
+        if (improved) { # improvement
             if (verbose) {
                 improve_ct <- (improve_ct %% length(improve_ch)) + 1L
-                cat(sprintf("% 5d     %s     %f\n", burst,
-                    improve_ch[improve_ct], cur_best_score))
+
+                cat(sprintf("% 5d     %s     %s\n", burst,
+                    improve_ch[improve_ct],
+                    fmt_score(cur_best_scores[, out_idx])))
             }
         } else if (verbose && burst %% report_int == 0) {
-            cat(sprintf("% 5d            %f\n", burst, cur_best_score))
+            cat(sprintf("% 5d            %s\n", burst,
+                        fmt_score(cur_best_scores[, out_idx])))
         }
 
         if (burst %% thin == 0) {
             idx <- burst %/% thin
-            out_mat[, idx] <- cur_best
-            scores[idx] <- cur_best_score
+            out_mat[, idx] <- cur_best[, out_idx]
+            scores[idx, ] <- cur_best_scores[, out_idx] * rescale
 
-            if (maximize && cur_best_score >= stop_at) break
-            if (!maximize && cur_best_score <= stop_at) break
+            if (any(colSums(cur_best_scores >= stop_at) == dim_score)) {
+                converged = TRUE
+                break
+            }
         }
     }
 
-    out_idx <- if (return_all) seq_len(idx) else idx
-    if (maximize)
-        converged <- cur_best_score >= stop_at
-    else
-        converged <- cur_best_score <= stop_at
-
-    storage.mode(out_mat) <- "integer"
-
-    out <- new_redist_plans(out_mat[, out_idx, drop = FALSE], map, "shortburst",
-        wgt = NULL, resampled = FALSE,
-        burst_size = burst_size,
-        n_bursts = burst,
-        backend = backend,
-        converged = converged,
-        score_fn = deparse(substitute(score_fn)))
-    out$score <- rep(scores[out_idx], each = ndists)
-
     if (return_all) {
+        out_idx <- seq_len(idx)
+        storage.mode(out_mat) <- "integer"
+
+        pareto_scores = t(cur_best_scores * rescale)
+        pareto_scores = pareto_scores[order(pareto_scores[, 1]), , drop=FALSE]
+
+        out <- new_redist_plans(out_mat[, out_idx, drop = FALSE], map, "shortburst",
+            wgt = NULL, resampled = FALSE,
+            n_bursts = burst,
+            backend = backend,
+            converged = converged,
+            pareto_front = cur_best,
+            pareto_scores = pareto_scores,
+            version = packageVersion("redist"),
+            score_fn = deparse(substitute(score_fn)))
+        score_mat = matrix(rep(scores[out_idx, ], each = ndists), ncol = dim_score)
+        colnames(score_mat) = colnames(scores)
+        out <- dplyr::mutate(out, as.data.frame(score_mat))
+        out$burst_size = rep(burst_sizes[out_idx], each = ndists)
+
         out <- add_reference(out, init_plan, "<init>")
-        out$score[1:ndists] <- scores[1]
+        idx_cols = ncol(out) - dim_score:1
+        out[1:ndists, idx_cols] <- matrix(rep(scores[1, ], each = ndists), ncol = dim_score)
+    } else {
+        out <- new_redist_plans(cur_best, map, "shortburst",
+                                wgt = NULL, resampled = FALSE,
+                                n_bursts = burst,
+                                backend = backend,
+                                converged = converged,
+                                version = packageVersion("redist"),
+                                score_fn = deparse(substitute(score_fn)))
+        score_mat = matrix(rep(t(cur_best_scores * rescale), each = ndists),
+                           ncol = dim_score)
+        colnames(score_mat) = colnames(scores)
+        out <- dplyr::mutate(out, as.data.frame(score_mat))
     }
 
     out
@@ -314,8 +387,9 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
 #'
 #' @param map A \code{\link{redist_map}} object.
 #'
-#' @return A scoring function of class `redist_scorer`. single numeric value, where larger values are better for `frac_kept`,
-#' `group_pct`, and `polsby_popper` and smaller values are better for `splits` and `pop_dev`.
+#' @return A scoring function of class `redist_scorer` which returns a single numeric value per plan.
+#' Larger values are generally better for `frac_kept`, `group_pct`, and `polsby_popper`
+#' and smaller values are better for `splits` and `pop_dev`.
 #'
 #' @examples
 #' \donttest{
@@ -327,6 +401,10 @@ redist_shortburst <- function(map, score_fn = NULL, stop_at = NULL,
 #' scorer_group_pct(iowa_map, dem_08, tot_08, k = 2)
 #' 1.5*scorer_frac_kept(iowa_map) + 0.4*scorer_status_quo(iowa_map)
 #' 1.5*scorer_frac_kept(iowa_map) + scorer_frac_kept(iowa_map)*scorer_status_quo(iowa_map)
+#' cbind(
+#'     comp = scorer_frac_kept(iowa_map),
+#'     sq = scorer_status_quo(iowa_map)
+#' )
 #' }
 #'
 #' @concept prepare
@@ -404,7 +482,7 @@ scorer_pop_dev <- function(map) {
 #' @export
 scorer_splits <- function(map, counties) {
     counties <- eval_tidy(enquo(counties), map)
-    counties <- as.integer(as.factor(counties))
+    counties <- vctrs::vec_group_id(counties)
 
     fn <- function(plans) {
         nd <- length(unique(plans[, 1]))
@@ -422,7 +500,7 @@ scorer_splits <- function(map, counties) {
 #' @export
 scorer_multisplits <- function(map, counties) {
     counties <- eval_tidy(enquo(counties), map)
-    counties <- as.integer(as.factor(counties))
+    counties <- vctrs::vec_group_id(counties)
 
     fn <- function(plans) {
         splits(plans, counties, attr(map, "ndists"), 2)/length(unique(counties))
@@ -434,15 +512,15 @@ scorer_multisplits <- function(map, counties) {
 #' @rdname scorers
 #' @order 6
 #'
-#' @param perim_df perimeter distance dataframe from \code{\link{redist.prep.polsbypopper}}
-#' @param areas area of each precinct (ie \code{st_area(map)})
+#' @param perim_df perimeter distance dataframe from [prep_perims()]
+#' @param areas area of each precinct (ie `st_area(map)`)
 #' @param m the m-th from the bottom Polsby Popper to return as the score. Defaults to 1,
 #' the minimum Polsby Popper score
 #'
 #' @export
 scorer_polsby_popper <- function(map, perim_df = NULL, areas = NULL, m = 1) {
     ndists <- attr(map, "ndists")
-    if (is.null(perim_df)) perim_df <- redist.prep.polsbypopper(map)
+    if (is.null(perim_df)) perim_df <- redistmetrics::prep_perims(map)
     if (is.null(areas)) areas <- sf::st_area(sf::st_geometry(map))
 
     fn <- function(plans) {
@@ -480,6 +558,48 @@ scorer_status_quo <- function(map, existing_plan = get_existing(map)) {
     fn
 }
 
+
+
+#' Combine scoring functions
+#'
+#' `redist_scorer` functions may be combined together to optimize along multiple
+#' dimensions. Rather than linearly combining multiple scorers to form a single
+#' objective as with [scorer-arith], these functions allow analysts to approximate
+#' the Pareto frontier for a set of scorers.
+#'
+#' @name scorer-combine
+#' @concept prepare
+#' @md
+#' @returns function of class redist_scorer. Will return a matrix with each
+#'   column containing every plan's scores for a particular scoring function.
+NULL
+
+#' @rdname scorer-combine
+#'
+#' @param ... a numeric or a `redist_scorer` function, from [`scorers`]
+#' @param deparse.level As in [cbind()].
+#'
+#' @export
+combine_scorers <- function(...) {
+    cbind(...)
+}
+
+#' @rdname scorer-combine
+#' @export
+cbind.redist_scorer <- function(..., deparse.level = 1) {
+    fns <- list(...)
+    stopifnot(all(sapply(fns, function(x) inherits(x, "redist_scorer"))))
+
+    fn <- function(plans) {
+        do.call(cbind, c(lapply(fns, function(fn) {
+            fn(plans)
+        }), list(deparse.level=deparse.level)))
+    }
+    class(fn) <- c("redist_scorer", "function")
+    fn
+}
+
+
 #' Scoring function arithmetic
 #'
 #' `redist_scorer` functions may be multiplied by constants and/or added
